@@ -414,9 +414,11 @@ add_unassigned_elements <- function(target_groups, init, N, K) {
   # assign elements that have no group (unfortunately, this "simple" task is quite difficult in general)
   if (length(table_assigned) != length(target_groups)) {
     table_assigned <- data.frame(K = 1:K, size = 0)
-    df <- as.data.frame(table(init))
+    df <- as.data.frame(table(init, useNA = "always"))
     together <- merge(table_assigned, df, by.x = "K", by.y = "init", all = TRUE)
-    table_assigned <- ifelse(is.na(together$Freq), 0, together$Freq)
+    together$K <- as.numeric(together$K)
+    together <- together[order(together$K), ]
+    table_assigned <- ifelse(is.na(together$Freq), 0, together$Freq)[-(K+1)]
   }
   freq_not_assigned <- target_groups - table_assigned
   init[is.na(init)] <- sample_(rep(1:K, freq_not_assigned))
@@ -465,8 +467,9 @@ remove_redundant_edges <- function(df) {
 
 # Function to solve optimal cannot_link constraints, used for the argument 
 # cannot_link in anticlustering().
-optimal_cannot_link <- function(N, K, target_groups, cannot_link, repetitions) {
-  repetitions <- ifelse(is.null(repetitions), 1, repetitions)
+# First function only returns a grouping for all elements that are fixated via the constraints.
+# Second function returns full groupings 
+optimal_cannot_link_reduced <- function(N, K, target_groups, cannot_link, repetitions) {
   all_nns_reordered <- reorder_edges(cannot_link)
   ilp <- k_coloring_ilp(all_nns_reordered, N, K, target_groups)
   # select solver: gurobi > symphony > lpSolve > Glpk
@@ -482,6 +485,12 @@ optimal_cannot_link <- function(N, K, target_groups, cannot_link, repetitions) {
     stop("The cannot-link constraints cannot be fulfilled.")
   }
   groups_fixated <- graph_coloring_to_group_vector(all_nns_reordered, solution$x, K, cannot_link, N)
+  groups_fixated
+}
+
+optimal_cannot_link <- function(N, K, target_groups, cannot_link, repetitions) {
+  repetitions <- ifelse(is.null(repetitions), 1, repetitions)
+  groups_fixated <- optimal_cannot_link_reduced(N, K, target_groups, cannot_link, repetitions)
   if (repetitions > 1) {
     groups <- t(replicate(repetitions, add_unassigned_elements(target_groups, groups_fixated, N, K)))
   } else {
