@@ -19,7 +19,7 @@
 #'     can be an object of class \code{dist} (e.g., returned by
 #'     \code{\link{dist}} or \code{\link{as.dist}}) or a \code{matrix}
 #'     where the entries of the upper and lower triangular matrix
-#'     represent pairwise dissimilarities.
+#'     represent pairwise dissimilarities. 
 #' @param K How many anticlusters should be created. Alternatively:
 #'     (a) A vector describing the size of each group, or (b) a vector
 #'     of length \code{nrow(x)} describing how elements are assigned
@@ -52,6 +52,9 @@
 #'     the same value in this vector cannot be assigned to the same anticluster.
 #' @param must_link A numeric vector of length \code{nrow(x)}. Elements having 
 #'     the same value in this vector are assigned to the same anticluster.
+#' @param blocks A vector, data.frame or matrix representing one
+#'     or several categorical variables. Anticlustering is done sequentially 
+#'     within blocks. See details.
 #'
 #' @return A vector of length N that assigns a group (i.e, a number
 #'     between 1 and \code{K}) to each input element.
@@ -111,10 +114,13 @@
 #' \code{\link{diversity_objective}}). Hence, anticlustering using the diversity 
 #' criterion maximizes between-group similarity
 #' by maximizing within-group heterogeneity (represented as the sum of all pairwise distances). 
-#' If it is computed on the basis of the Euclidean distance (which is the default
+#' In the publication that introduced
+#' the \code{anticlust} package (Papenberg & Klau, 2021), we used the term "anticluster 
+#' editing" to refer to the maximization of the diversity, because the reversed 
+#' procedure - minimizing the diversity - is also known as "cluster editing".
+#' If the diversity is computed on the basis of the Euclidean distance (which is the default
 #' behaviour if \code{x} is a feature matrix), the diversity is an all rounder objective that 
-#' tends to equate all distribution 
-#' characteristics between groups (such as means, variances, ...). 
+#' tends to equate all distribution characteristics between groups (such as means, variances, ...). 
 #' Note that the equivalence of within-group heterogeneity and between-group similarity only
 #' holds for equal-sized groups. For unequal-sized groups, it is recommended to
 #' use a different objective when striving for overall between-group similarity,
@@ -122,10 +128,7 @@
 #' was introduced in version 0.8.6, and it is more useful if groups are not 
 #' equal-sized. The average diversity normalizes the sum of intra-cluster distances 
 #' by group size. If all groups are equal-sized, it is equivalent to the 
-#' regular diversity. In the publication that introduces
-#' the \code{anticlust} package (Papenberg & Klau, 2021), we used the term "anticluster 
-#' editing" to refer to the maximization of the diversity, because the reversed 
-#' procedure - minimizing the diversity - is also known as "cluster editing". 
+#' regular diversity. 
 #' 
 #' The "dispersion" is the minimum distance between any two elements
 #' that are part of the same cluster; maximization of this objective
@@ -137,21 +140,52 @@
 #' same time, see the function
 #' \code{\link{bicriterion_anticlustering}}.
 #'
-#' If the data input \code{x} is a feature matrix (that is: each row
-#' is a "case" and each column is a "variable") and the option
-#' \code{objective = "diversity"} or \code{objective = "dispersion"} is used, 
-#' the Euclidean distance is computed as the basic unit of the objectives. If
-#' a different measure of dissimilarity is preferred, you may pass a
-#' self-generated dissimilarity matrix via the argument \code{x}.
-#'
 #' In the standard case, groups of equal size are generated. Adjust
 #' the argument \code{K} to create groups of different size (see
 #' Examples).
 #' 
+#' 
+#' \strong{Data input}
+#' 
+#' The data input \code{x} is usually a feature matrix where each each row
+#' is a "case" and each column is a "variable". The default
+#' \code{objective = "diversity"} or the \code{objective = "dispersion"} are computed
+#' using a dissimilarity matrix as input, however. The default behaviour for 
+#' feature matrices is to compute the Euclidean distance as the 
+#' basic unit of these objectives, using a call to \code{\link{dist}}. 
+#' The combination \code{objective = "diversity"} + 
+#' Euclidean distance was evaluated by us (e.g., Papenberg and Klau, 2021; 
+#' Papenberg, 2024; Papenberg, Wang, et al., 2025) and is generally considered 
+#' to be a good default for equal-sized groups. For unequal-sized groups, prefer 
+#' \code{objective = "average-diversity"}. 
+#' 
+#' If a different type of dissimilarity measure is preferred (e.g., the Gower distance), 
+#' you may pass a self-generated dissimilarity matrix via the argument \code{x}. This dissimilarity
+#' matrix can be an object of class \code{dist} (e.g., returned by \code{\link{dist}} 
+#' or \code{\link{as.dist}}) or a \code{matrix} where the entries of the upper 
+#' and lower triangular matrix represent pairwise dissimilarities. If the input
+#' is a matrix rather than of class \code{dist}, the function internally tests if
+#' the input is a dissimilarity matrix (because a \code{matrix} could also be a 
+#' feature matrix). It assumes the input is a dissimilarity matrix if the 
+#' upper and lower triangular consists of the same values, and if the diagonal 
+#' only consists of the same values. If the input is a \code{data.frame},
+#' this function always assumes that it is a feature matrix. In a \code{data.frame},
+#' each column must be either \code{numeric} or \code{factor} (the latter for 
+#' categorical variables).
+#' 
 #' As of version 0.8.12, this function supports handling of missing values (\code{NA}). 
-#' In this case the function \code{\link[stats]{dist}} handles the (\code{NA}) when converting
-#' features to pairwise distances. 
-#'
+#' In this case the function \code{\link[stats]{dist}} handles the \code{NA}s when converting
+#' features to pairwise distances. Note that \code{\link[stats]{dist}} always handles
+#' missing values even for objectives that are usually computed using a feature 
+#' matrix as input (i.e., \code{objective = "variance"} and \code{objective = "kplus"}).
+#' In this case, we exploit an equivalence between the \code{objective = "variance"} and
+#' the \code{objective = "average-diversity"}: When the squared Euclidean distance
+#' \code{objective = "variance"} and the \code{objective = "average-diversity"}
+#' are actually equivalent, so we can optimize the \code{objective = "average-diversity"}
+#' in order to perform k-means (and thus, k-plus) anticlustering. See Papenberg, 
+#' Breuer, et al. (2025) for details.
+#' 
+#' 
 #' \strong{Algorithms for anticlustering}
 #'
 #' By default, a heuristic method is employed for anticlustering: the
@@ -234,7 +268,7 @@
 #' 
 #' \strong{Categorical variables}
 #'
-#' There are two ways to balance categorical variables among anticlusters (also 
+#' There are several ways to balance categorical variables among anticlusters (also 
 #' see the package vignette "Using categorical variables with anticlustering").
 #' The first way is to treat them as "hard constraints" via the argument 
 #' \code{categories} (see Papenberg & Klau, 2021). If done so, balancing the 
@@ -262,6 +296,15 @@
 #' what users need. In this case, I still recommend using \code{\link{categories_to_binary}} 
 #' manually.
 #' 
+#' The argument \code{blocks} (available as of version 0.8.13) is similar 
+#' to \code{categories}. Anticlustering is repeated within each stratum
+#' (i.e., each levels or combination of levels of \code{blocks} -- as in \code{categories}
+#' mulitple variables are merged into a single variable to form strata). Anticlustering
+#' in later blocks "remembers" the assignment of previous blocks. This way, 
+#' overall balance as well as balance within blocks is maximized. This is unlike 
+#' when using \code{categories}, which only maximizes overall balance, but not 
+#' specifically within strata. 
+#' 
 #' \strong{Anticlustering with constraints}
 #' 
 #' Versions 0.8.6 and 0.8.7 of anticlust introduced the possibility to induce
@@ -434,11 +477,11 @@
 #' Maximizing Between-Group Similarity. British Journal of Mathematical and 
 #' Statistical Psychology, 77(1), 80-102. https://doi.org/10.1111/bmsp.12315
 #' 
-#' Papenberg, M., Wang, C., Diop, M., Bukhari, S. H., Oskotsky, B., Davidson, 
-#' B. R., Vo, K. C., Liu, B., Irwin, J. C., Combes, A., Gaudilliere, B., 
-#' Li, J., Stevenson, D. K., Klau, G. W., Giudice, L. C., Sirota, M., 
-#' & Oskotsky, T. T. (2025). Anticlustering for sample allocation to minimize 
-#' batch effects. bioRxiv. https://doi.org/10.1101/2025.03.03.641320
+#' Papenberg, M., Wang, C., Diop, M., Bukhari, S. H., Oskotsky, B., Davidson, B. R., 
+#' Vo, K. C., Liu, B., Irwin, J. C., Combes, A., Gaudilliere, B., Li, J., Stevenson, D. K., 
+#' Klau, G. W., Giudice, L. C., Sirota, M., & Oskotsky, T. T. (2025). Anticlustering
+#' for sample allocation to minimize batch effects. Cell Reports Methods, 5(8), 
+#' 101137. https://doi.org/10.1016/j.crmeth.2025.101137
 #'
 #' Späth, H. (1986). Anticlustering: Maximizing the variance criterion.
 #' Control and Cybernetics, 15, 213-218.
@@ -456,14 +499,15 @@
 anticlustering <- function(x, K, objective = "diversity", method = "exchange",
                            preclustering = FALSE, categories = NULL, 
                            repetitions = NULL, standardize = FALSE, cannot_link = NULL,
-                           must_link = NULL) {
+                           must_link = NULL, blocks = NULL) {
 
 
   ## Get data into required format
   input_validation_anticlustering(x, K, objective, method, preclustering, 
                                   categories, repetitions, standardize, cannot_link,
-                                  must_link)
+                                  must_link, blocks)
 
+  input_is_a_feature_matrix <- !is_distance_matrix(x) || inherits(x, "data.frame")
   
   # Preclustering and categorical constraints are both processed in the
   # variable `categories` after this step:
@@ -472,7 +516,7 @@ anticlustering <- function(x, K, objective = "diversity", method = "exchange",
   categories <- get_categorical_constraints(x, K, preclustering, categories)
   
   # Convert input into usable features for anticlustering, if feature matrix is passed and not custom objective used
-  if (!is_distance_matrix(x) && !is.function(objective)) {
+  if (input_is_a_feature_matrix && !is.function(objective)) {
     x <- get_anticlustering_features(x, objective, standardize)
     if (any(rowSums(is.na(x)) == ncol(x))) {
       stop("Some observations only consist of NA, I cannot deal with this.")
@@ -509,6 +553,18 @@ anticlustering <- function(x, K, objective = "diversity", method = "exchange",
     }
   }
   
+  ####### Data handling is done; now call the required algorithm
+  
+  if (argument_exists(blocks)) {
+    return(blocked_anticlustering(
+      x, objective = objective, 
+      method = method, 
+      categories = categories, K = K, 
+      cannot_link = cannot_link, 
+      blocks = blocks
+    ))
+  }
+  
   if (method == "3phase") {
     if (length(K) == NUMBER_OF_ANTICLUSTERS) {
       clusters <- K
